!-------------------------------------------------------------------------------

!                      Code_Saturne version 2.0.0-rc2
!                      --------------------------

!     This file is part of the Code_Saturne Kernel, element of the
!     Code_Saturne CFD tool.

!     Copyright (C) 1998-2009 EDF S.A., France

!     contact: saturne-support@edf.fr

!     The Code_Saturne Kernel is free software; you can redistribute it
!     and/or modify it under the terms of the GNU General Public License
!     as published by the Free Software Foundation; either version 2 of
!     the License, or (at your option) any later version.

!     The Code_Saturne Kernel is distributed in the hope that it will be
!     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
!     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.

!     You should have received a copy of the GNU General Public License
!     along with the Code_Saturne Kernel; if not, write to the
!     Free Software Foundation, Inc.,
!     51 Franklin St, Fifth Floor,
!     Boston, MA  02110-1301  USA

!-------------------------------------------------------------------------------

subroutine uselcl &
!================

 ( idbia0 , idbra0 ,                                              &
   ndim   , ncelet , ncel   , nfac   , nfabor , nfml   , nprfml , &
   nnod   , lndfac , lndfbr , ncelbr ,                            &
   nvar   , nscal  , nphas  ,                                     &
   nideve , nrdeve , nituse , nrtuse ,                            &
   ifacel , ifabor , ifmfbr , ifmcel , iprfml , maxelt , lstelt , &
   ipnfac , nodfac , ipnfbr , nodfbr ,                            &
   icodcl , itrifb , itypfb , izfppp ,                            &
   idevel , ituser , ia     ,                                     &
   xyzcen , surfac , surfbo , cdgfac , cdgfbo , xyznod , volume , &
   dt     , rtp    , rtpa   , propce , propfa , propfb ,          &
   coefa  , coefb  , rcodcl ,                                     &
   w1     , w2     , w3     , w4     , w5     , w6     , coefu  , &
   rdevel , rtuser , ra     )

!===============================================================================
! Purpose
! --------
!
!    User subroutine
!
!    THIS ROUTINE IS UNAVOIDABLE
!    ===========================
!
!    Fill boundary conditions arrays (icodcl, rcodcl) for variables
!    used in the electric modules of the code
!    (Joule effect by direct conduction, Electric arc and ionic mobility).
!
!    The variable are the standart one (enthalpy, velocity )and the "electric variables".
!
!    The "electric" variable are :
!     - at first and always the electric potential (real component in the case of complex potential)
!     - in option you could have imaginary component of the electrical potential
!       (Joule effect by direct conduction)
!
!     - 3 components of the vector potential A (for Electric arc module)
!
!
!
! Introduction
! ============

! Here one defines boundary conditions on a per-face basis.

! Boundary faces may be identified using the 'getfbr' subroutine.
! The syntax of this subroutine is described in the 'usclim' subroutine,
! but a more thorough description can be found in the user guide.


! Boundary condition types
! ========================

! Boundary conditions setup for standard variables (pressure, velocity,
! turbulence, scalars) is described precisely in the 'usclim' subroutine.

! Detailed explanation will be found in the theory guide.

!-------------------------------------------------------------------------------
! Arguments
!__________________.____._____.________________________________________________.
! name             !type!mode ! role                                           !
!__________________!____!_____!________________________________________________!
! idbia0           ! i  ! <-- ! number of first free position in ia            !
! idbra0           ! i  ! <-- ! number of first free position in ra            !
! ndim             ! i  ! <-- ! spatial dimension                              !
! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
! ncel             ! i  ! <-- ! number of cells                                !
! nfac             ! i  ! <-- ! number of interior faces                       !
! nfabor           ! i  ! <-- ! number of boundary faces                       !
! nfml             ! i  ! <-- ! number of families (group classes)             !
! nprfml           ! i  ! <-- ! number of properties per family (group class)  !
! nnod             ! i  ! <-- ! number of vertices                             !
! lndfac           ! i  ! <-- ! size of nodfac indexed array                   !
! lndfbr           ! i  ! <-- ! size of nodfbr indexed array                   !
! ncelbr           ! i  ! <-- ! number of cells with faces on boundary         !
! nvar             ! i  ! <-- ! total number of variables                      !
! nscal            ! i  ! <-- ! total number of scalars                        !
! nphas            ! i  ! <-- ! number of phases                               !
! nideve, nrdeve   ! i  ! <-- ! sizes of idevel and rdevel arrays              !
! nituse, nrtuse   ! i  ! <-- ! sizes of ituser and rtuser arrays              !
! ifacel(2, nfac)  ! ia ! <-- ! interior faces -> cells connectivity           !
! ifabor(nfabor)   ! ia ! <-- ! boundary faces -> cells connectivity           !
! ifmfbr(nfabor)   ! ia ! <-- ! boundary face family numbers                   !
! ifmcel(ncelet)   ! ia ! <-- ! cell family numbers                            !
! iprfml           ! ia ! <-- ! property numbers per family                    !
!  (nfml, nprfml)  !    !     !                                                !
! maxelt           !  e ! <-- ! max number of cells and faces (int/boundary)   !
! lstelt(maxelt)   ! ia ! --- ! work array                                     !
! ipnfac(nfac+1)   ! ia ! <-- ! interior faces -> vertices index (optional)    !
! nodfac(lndfac)   ! ia ! <-- ! interior faces -> vertices list (optional)     !
! ipnfbr(nfabor+1) ! ia ! <-- ! boundary faces -> vertices index (optional)    !
! nodfac(lndfbr)   ! ia ! <-- ! boundary faces -> vertices list (optional)     !
! icodcl           ! ia ! --> ! boundary condition code                        !
!  (nfabor, nvar)  !    !     ! = 1  -> Dirichlet                              !
!                  !    !     ! = 2  -> flux density                           !
!                  !    !     ! = 4  -> sliding wall and u.n=0 (velocity)      !
!                  !    !     ! = 5  -> friction and u.n=0 (velocity)          !
!                  !    !     ! = 6  -> roughness and u.n=0 (velocity)         !
!                  !    !     ! = 9  -> free inlet/outlet (velocity)           !
!                  !    !     !         inflowing possibly blocked             !
! itrifb(nfabor    ! ia ! <-- ! indirection for boundary faces ordering)       !
!  (nfabor, nphas) !    !     !                                                !
! itypfb           ! ia ! --> ! boundary face types                            !
!  (nfabor, nphas) !    !     !                                                !
! idevel(nideve)   ! ia ! <-- ! integer work array for temporary developpement !
! ituser(nituse    ! ia ! <-- ! user-reserved integer work array               !
! ia(*)            ! ia ! --- ! main integer work array                        !
! xyzcen           ! ra ! <-- ! cell centers                                   !
!  (ndim, ncelet)  !    !     !                                                !
! surfac           ! ra ! <-- ! interior faces surface vectors                 !
!  (ndim, nfac)    !    !     !                                                !
! surfbo           ! ra ! <-- ! boundary faces surface vectors                 !
!  (ndim, nfavor)  !    !     !                                                !
! cdgfac           ! ra ! <-- ! interior faces centers of gravity              !
!  (ndim, nfac)    !    !     !                                                !
! cdgfbo           ! ra ! <-- ! boundary faces centers of gravity              !
!  (ndim, nfabor)  !    !     !                                                !
! xyznod           ! ra ! <-- ! vertex coordinates (optional)                  !
!  (ndim, nnod)    !    !     !                                                !
! volume(ncelet)   ! ra ! <-- ! cell volumes                                   !
! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
!  (ncelet, *)     !    !     !  (at current and preceding time steps)         !
! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
! propfa(nfac, *)  ! ra ! <-- ! physical properties at interior face centers   !
! propfb(nfabor, *)! ra ! <-- ! physical properties at boundary face centers   !
! coefa, coefb     ! ra ! <-- ! boundary conditions                            !
!  (nfabor, *)     !    !     !                                                !
! rcodcl           ! ra ! --> ! boundary condition values                      !
!                  !    !     ! rcodcl(1) = Dirichlet value                    !
!                  !    !     ! rcodcl(2) = exterior exchange coefficient      !
!                  !    !     !  (infinite if no exchange)                     !
!                  !    !     ! rcodcl(3) = flux density value                 !
!                  !    !     !  (negative for gain) in w/m2 or                !
!                  !    !     !  roughness height (m) if icodcl=6              !
!                  !    !     ! for velocities           ( vistl+visct)*gradu  !
!                  !    !     ! for pressure                         dt*gradp  !
!                  !    !     ! for scalars    cp*(viscls+visct/sigmas)*gradt  !
! w1,2,3,4,5,6     ! ra ! --- ! work arrays                                    !
!  (ncelet)        !    !     !  (computation of pressure gradient)            !
! coefu            ! ra ! --- ! work array                                     !
!  (nfabor, 3)     !    !     !  (computation of pressure gradient)            !
! rdevel(nrdeve)   ! ra ! <-> ! real work array for temporary developpement    !
! rtuser(nituse    ! ra ! <-- ! user-reserved real work array                  !
! ra(*)            ! ra ! --- ! main real work array                           !
!__________________!____!_____!________________________________________________!

!     Type: i (integer), r (real), s (string), a (array), l (logical),
!           and composite types (ex: ra real array)
!     mode: <-- input, --> output, <-> modifies data, --- work array
!===============================================================================

implicit none

!===============================================================================
!    Common blocks
!===============================================================================

include "paramx.h"
include "numvar.h"
include "optcal.h"
include "cstphy.h"
include "cstnum.h"
include "entsor.h"
include "ppppar.h"
include "ppthch.h"
include "ppincl.h"
include "elincl.h"

!===============================================================================

! Arguments

integer          idbia0 , idbra0
integer          ndim   , ncelet , ncel   , nfac   , nfabor
integer          nfml   , nprfml
integer          nnod   , lndfac , lndfbr , ncelbr
integer          nvar   , nscal  , nphas
integer          nideve , nrdeve , nituse , nrtuse

integer          ifacel(2,nfac) , ifabor(nfabor)
integer          ifmfbr(nfabor) , ifmcel(ncelet)
integer          iprfml(nfml,nprfml)
integer          maxelt, lstelt(maxelt)
integer          ipnfac(nfac+1), nodfac(lndfac)
integer          ipnfbr(nfabor+1), nodfbr(lndfbr)
integer          icodcl(nfabor,nvar)
integer          itrifb(nfabor,nphas), itypfb(nfabor,nphas)
integer          izfppp(nfabor)
integer          idevel(nideve), ituser(nituse), ia(*)

double precision xyzcen(ndim,ncelet)
double precision surfac(ndim,nfac), surfbo(ndim,nfabor)
double precision cdgfac(ndim,nfac), cdgfbo(ndim,nfabor)
double precision xyznod(ndim,nnod), volume(ncelet)
double precision dt(ncelet), rtp(ncelet,*), rtpa(ncelet,*)
double precision propce(ncelet,*)
double precision propfa(nfac,*), propfb(nfabor,*)
double precision coefa(nfabor,*), coefb(nfabor,*)
double precision rcodcl(nfabor,nvar,3)
double precision w1(ncelet),w2(ncelet),w3(ncelet)
double precision w4(ncelet),w5(ncelet),w6(ncelet)
double precision coefu(nfabor,ndim)
double precision rdevel(nrdeve), rtuser(nrtuse), ra(*)

! Local variables

integer          idebia, idebra
integer          ifac, ii, iphas , iel
integer          idim
integer          izone,iesp
integer          ilelt, nlelt

double precision uref2, d2s3
double precision rhomoy, dhy, ustar2
double precision xkent, xeent
double precision z1   , z2


double precision rayon, temp, enth
integer           MUN, mode, iesp2
double precision COEFE(NGAZEM)





!===============================================================================
! 1.  Initialization
!===============================================================================

idebia = idbia0
idebra = idbra0

d2s3 = 2.d0/3.d0


!===============================================================================
! 2.  Assign boundary conditions to boundary faces here

!     One may use selection criteria to filter boundary case subsets
!       Loop on faces from a subset
!         Set the boundary condition for each face
!===============================================================================

iphas = 1


 CALL GETFBR('all[]',NLELT,LSTELT)
!==========

do ilelt = 1, nlelt

  ifac = lstelt(ilelt)
  iel  = ifabor(ifac)
  rayon= SQRT( XYZCEN(1,iel)*XYZCEN(1,iel) +XYZCEN(2,iel)*XYZCEN(2,iel) )

  ! Entrée 1
  if (XYZCEN(3,iel).gt.0.0d0 .and. abs(surfbo(1,ifac)) .lt. 1.0d-8 .and. abs(surfbo(2,ifac)) .lt. 1.0d-8 ) then

        izone = 1
        izfppp(ifac) = izone
        itypfb(ifac,iphas) = ientre


        !! Potentiel reel
        ii = ipotr
        icodcl(ifac,isca(ii))   = 1
        rcodcl(ifac,isca(ii),1) = 0.0

        !! Potentiel vecteur
        do idim= 1,ndimve
           ii = ipotva(idim)
           icodcl(ifac,isca(ii))   = 3
           rcodcl(ifac,isca(ii),3) = 0.d0
        enddo     

        !! Vitesse
        icodcl(ifac,iu(iphas))   = 1
        rcodcl(ifac,iu(iphas),1) = 0.0d0
        icodcl(ifac,iv(iphas))   = 1
        rcodcl(ifac,iv(iphas),1) = 0.0d0
        icodcl(ifac,iw(iphas))   = 1
        rcodcl(ifac,iw(iphas),1) = -5.0d0

        !! Fraction massique
        icodcl(ifac,isca(iycoel(1))) = 1
        rcodcl(ifac,isca(iycoel(1)),1) = 1.0d0

        !! Enthalphie
        ii = ihm
        icodcl(ifac,isca(ii))   = 1
        mode = -1
        coefe(1)=1.0d0 !rtp(iel,isca(iycoel(1)))
        coefe(2)=0.0d0 !1.0d0-rtp(iel,isca(iycoel(1)))
        call elthht(mode,ngazg,coefe,enth,10000.0D0)
        rcodcl(ifac,isca(ii),1) = enth


  !! Sortie
  elseif (XYZCEN(3,iel).lt.0.0d0 .and. abs(surfbo(1,ifac)) .lt. 1.0d-8 .and. abs(surfbo(2,ifac)) .lt. 1.0d-8  ) then

        izone = 2
        izfppp(ifac) = izone
        itypfb(ifac,iphas) = isolib
        
        ! Fraction massique
        !icodcl(ifac,isca(iycoel(1))) = 1
        !rcodcl(ifac,isca(iycoel(1)),1) = 1.0d0

        !! Enthalphie
        !ii = ihm
        !mode = -1
        !coefe(1)=rtp(iel,isca(iycoel(1)))
        !coefe(2)=1.0d0-rtp(iel,isca(iycoel(1)))
        !call elthht(mode,ngazg,coefe,enth,10000.0D0)
        !icodcl(ifac,isca(ii))   = 1
        !rcodcl(ifac,isca(ii),1) = enth


 !! Entrée 2
  elseif (XYZCEN(3,iel).gt.0.10d0 .and. XYZCEN(3,iel).lt.0.12d0 .and. XYZCEN(2,iel).gt.0.03d0 ) then

        izone = 3
        izfppp(ifac) = izone
        itypfb(ifac,iphas) = ientre


        !! Potentiel reel
        ii = ipotr
        icodcl(ifac,isca(ii))   = 1
        rcodcl(ifac,isca(ii),1) = 0.0

        !! Potentiel vecteur
        do idim= 1,ndimve
           ii = ipotva(idim)
           icodcl(ifac,isca(ii))   = 3
           rcodcl(ifac,isca(ii),3) = 0.d0
        enddo     

        !! Vitesse
        icodcl(ifac,iu(iphas))   = 1
        rcodcl(ifac,iu(iphas),1) = 0.0d0
        icodcl(ifac,iv(iphas))   = 1
        rcodcl(ifac,iv(iphas),1) = -15.0d0
        icodcl(ifac,iw(iphas))   = 1
        rcodcl(ifac,iw(iphas),1) = 0.0d0

        !! Fraction massique
        icodcl(ifac,isca(iycoel(1))) = 1
        rcodcl(ifac,isca(iycoel(1)),1) = 0.0d0

        !! Enthalphie
        ii = ihm
        icodcl(ifac,isca(ii))   = 1
        mode = -1
        coefe(1)=0.0d0 !rtp(iel,isca(iycoel(1)))
        coefe(2)=1.0d0 !1.0d0-rtp(iel,isca(iycoel(1)))
        call elthht(mode,ngazg,coefe,enth,10000.0D0)
        rcodcl(ifac,isca(ii),1) = enth

  !! Mur
  else 

        izone = 6
        izfppp(ifac) = izone
        itypfb(ifac,iphas) = iparoi

        icodcl(ifac,isca(iycoel(1))) = 3
        rcodcl(ifac,isca(iycoel(1)),3) = 0.0d0

        !! Enthalphie
        ii = ihm
        icodcl(ifac,isca(ii))   = 1
        mode = -1
        coefe(1)=rtp(iel,isca(iycoel(1)))
        coefe(2)=1.0d0-rtp(iel,isca(iycoel(1)))
        call elthht(mode,ngazg,coefe,enth,10000.0D0)
        rcodcl(ifac,isca(ii),1) = enth

   endif

enddo


!----
! FORMATS
!----

!----
! FIN
!----

return
end subroutine
